@@ -2,10 +2,9 @@
 /**
  * Horde_CLI:: API for basic command-line functionality/checks.
  *
- * $Horde: framework/CLI/CLI.php,v 1.42.6.13 2006/05/31 12:31:00 jan Exp $
+ * $Horde: framework/CLI/CLI.php,v 1.42.6.24 2008/07/16 17:40:46 jan Exp $
  *
- * Copyright 2003-2006 Chuck Hagenbuch <chuck@horde.org>
- * Copyright 2003-2006 Jan Schneider <jan@horde.org>
+ * Copyright 2003-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -31,6 +30,13 @@ class Horde_CLI {
      */
     var $_newline;
 
+    /**
+     * The string to use for clearing the screen.
+     *
+     * @var string
+     */
+    var $_clearscreen = '';
+
     /**
      * The indent string to use.
      *
@@ -68,6 +74,47 @@ class Horde_CLI {
     var $_yellow_end   = '';
     var $_blue_end     = '';
 
+    /**
+     * Terminal foreground color codes. Not used yet.
+     * @var array
+     */
+    var $_terminalForegrounds = array(
+        'normal'        => "\x1B[0m",
+        'black'         => "\x1B[0m",
+        'bold'          => "\x1b[1m",
+        'red'           => "\x1B[31m",
+        'green'         => "\x1B[32m",
+        'brown'         => "\x1B[33m",
+        'blue'          => "\x1B[34m",
+        'magenta'       => "\x1B[35m",
+        'cyan'          => "\x1B[36m",
+        'lightgray'     => "\x1B[37m",
+        'white'         => "\x1B[1m\x1B[37m",
+        'darkgray'      => "\x1B[1m\x1B[0m",
+        'lightred'      => "\x1B[1m\x1B[31m",
+        'lightgreen'    => "\x1B[1m\x1B[32m",
+        'yellow'        => "\x1B[1m\x1B[33m",
+        'lightblue'     => "\x1B[1m\x1B[34m",
+        'lightmagenta'  => "\x1B[1m\x1B[35m",
+        'lightcyan'     => "\x1B[1m\x1B[36m",
+    );
+
+    /**
+     * Terminal background color codes. Not used yet.
+     * @var array
+     */
+    var $_terminalBackgrounds = array(
+        'normal'    => "\x1B[0m",
+        'black'     => "\x1B[0m",
+        'red'       => "\x1B[41m",
+        'green'     => "\x1B[42m",
+        'brown'     => "\x1B[43m",
+        'blue'      => "\x1B[44m",
+        'magenta'   => "\x1B[45m",
+        'cyan'      => "\x1B[46m",
+        'lightgray' => "\x1B[47m",
+    );
+
     /**
      * Returns a single instance of the Horde_CLI class.
      */
@@ -83,6 +130,10 @@ class Horde_CLI {
     /**
      * Detect the current environment (web server or console) and sets
      * internal values accordingly.
+     *
+     * The constructor must not be called after init(). Either use the
+     * singleton() method to retrieve a Horde_CLI object, or don't call init()
+     * statically.
      */
     function Horde_CLI()
     {
@@ -95,6 +146,7 @@ class Horde_CLI {
             $term = getenv('TERM');
             if ($term) {
                 if (preg_match('/^(xterm|vt220|linux)/', $term)) {
+                    $this->_clearscreen  = "\x1b[2J\x1b[H";
                     $this->_bold_start   = "\x1b[1m";
                     $this->_red_start    = "\x1b[01;31m";
                     $this->_green_start  = "\x1b[01;32m";
@@ -102,6 +154,7 @@ class Horde_CLI {
                     $this->_blue_start   = "\x1b[01;34m";
                     $this->_bold_end = $this->_red_end = $this->_green_end = $this->_yellow_end = $this->_blue_end = "\x1b[0m";
                 } elseif (preg_match('/^vt100/', $term)) {
+                    $this->_clearscreen  = "\x1b[2J\x1b[H";
                     $this->_bold_start = "\x1b[1m";
                     $this->_bold_end   = "\x1b[0m";
                 }
@@ -136,6 +189,16 @@ class Horde_CLI {
         }
     }
 
+    /**
+     * Clears the entire screen, if possible.
+     *
+     * @since Horde 3.2
+     */
+    function clearScreen()
+    {
+        echo $this->_clearscreen;
+    }
+
     /**
      * Returns the indented string.
      *
@@ -205,7 +268,8 @@ class Horde_CLI {
      */
     function message($message, $type = 'cli.message')
     {
-        $message = wordwrap($message, 68, "\n           ", true);
+        $message = wordwrap(str_replace("\n", "\n           ", $message),
+                            68, "\n           ", true);
 
         switch ($type) {
         case 'cli.error':
@@ -241,7 +305,7 @@ class Horde_CLI {
         $this->writeln($this->red($error));
         $this->writeln();
         $this->writeln($this->red('===================='));
-        exit;
+        exit(1);
     }
 
     /**
@@ -281,9 +345,10 @@ class Horde_CLI {
                 if (isset($choices[$response])) {
                     return $response;
                 } else {
-                    $this->writeln(sprintf(_("\"%s\" is not a valid choice."), $response));
+                    $this->writeln($this->red(sprintf(_("\"%s\" is not a valid choice."), $response)));
                 }
             } else {
+                @ob_flush();
                 $response = trim(fgets(STDIN));
                 if ($response === '' && $default !== null) {
                     $response = $default;
@@ -315,6 +380,10 @@ class Horde_CLI {
      * none. Also initialize a few variables in $_SERVER that aren't present
      * from the CLI.
      *
+     * You must not call init() statically before calling the constructor.
+     * Either use the singleton() method to retrieve a Horde_CLI object after
+     * calling init(), or don't call init() statically.
+     *
      * @static
      */
     function init()
@@ -352,11 +421,11 @@ class Horde_CLI {
      */
     function runningFromCLI()
     {
-        $sapi = php_sapi_name();
-        if ($sapi == 'cli') {
+        if (PHP_SAPI == 'cli') {
             return true;
         } else {
-            return $sapi == 'cgi' && empty($_SERVER['SERVER_NAME']);
+            return (PHP_SAPI == 'cgi' || PHP_SAPI == 'cgi-fcgi') &&
+                empty($_SERVER['SERVER_NAME']);
         }
     }
 
