@@ -47,9 +47,9 @@ define('MIME_STRING', 2);
 /**
  * The MIME:: class provides methods for dealing with MIME standards.
  *
- * $Horde: framework/MIME/MIME.php,v 1.139.4.4 2005/03/07 14:22:06 jan Exp $
+ * $Horde: framework/MIME/MIME.php,v 1.139.4.45 2008/10/26 19:41:01 slusarz Exp $
  *
- * Copyright 1999-2005 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 1999-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -63,7 +63,7 @@ class MIME {
     /**
      * A listing of the allowed MIME types.
      *
-     * @var array $mime_types
+     * @var array
      */
     var $mime_types = array(
         TYPETEXT => 'text',
@@ -80,7 +80,7 @@ class MIME {
     /**
      * A listing of the allowed MIME encodings.
      *
-     * @var array $mime_encodings
+     * @var array
      */
     var $mime_encodings = array(
         ENC7BIT => '7bit',
@@ -92,31 +92,27 @@ class MIME {
     );
 
     /**
-     * Filter for RFC822.
+     * Determines if a string contains 8-bit (non US-ASCII) characters.
      *
-     * @var array $rfc822_filter
-     */
-    var $rfc822_filter = "()<>@,;:\\\"[]\1\2\3\4\5\6\7\10\11\12\13\14\15\16\17\20\21\22\23\24\25\26\27\30\31\32\33\34\35\36\37\177";
-
-    /**
-     * Determines if a string contains 8-bit characters.
-     *
-     * @access public
-     *
-     * @param string $string  The string to check.
+     * @param string $string   The string to check.
+     * @param string $charset  The charset of the string. Defaults to
+     *                         US-ASCII. Since Horde 3.2.2.
      *
      * @return boolean  True if it does, false if it doesn't.
      */
-    function is8bit($string)
+    function is8bit($string, $charset = null)
     {
-        return (is_string($string) && preg_match('/[\x80-\xff]/', $string));
+        /* ISO-2022-JP is a 7bit charset, but it is an 8bit representation so
+         * it needs to be entirely encoded. */
+        return is_string($string) &&
+               ((stristr('iso-2022-jp', $charset) &&
+                (strstr($string, "\x1b\$B"))) ||
+                preg_match('/[\x80-\xff]/', $string));
     }
 
     /**
      * Encodes a string containing non-ASCII characters according to RFC 2047.
      *
-     * @access public
-     *
      * @param string $text     The text to encode.
      * @param string $charset  The character set of the text.
      *
@@ -125,25 +121,26 @@ class MIME {
      */
     function encode($text, $charset = null)
     {
-        /* Return if nothing needs to be encoded. */
-        if (!MIME::is8bit($text)) {
-            return $text;
-        }
-
         if (is_null($charset)) {
+            require_once 'Horde/NLS.php';
             $charset = NLS::getCharset();
         }
-
         $charset = String::lower($charset);
+
         $line = '';
 
+        /* Return if nothing needs to be encoded. */
+        if (($charset == 'us-ascii') || !MIME::is8bit($text, $charset)) {
+            return $text;
+        }
+
         /* Get the list of elements in the string. */
         $size = preg_match_all('/([^\s]+)([\s]*)/', $text, $matches, PREG_SET_ORDER);
 
         foreach ($matches as $key => $val) {
-            if (MIME::is8bit($val[1])) {
+            if (MIME::is8bit($val[1], $charset)) {
                 if ((($key + 1) < $size) &&
-                    MIME::is8bit($matches[$key + 1][1])) {
+                    MIME::is8bit($matches[$key + 1][1], $charset)) {
                     $line .= MIME::_encode($val[1] . $val[2], $charset) . ' ';
                 } else {
                     $line .= MIME::_encode($val[1], $charset) . $val[2];
@@ -169,25 +166,34 @@ class MIME {
      */
     function _encode($text, $charset)
     {
-        $char_len = strlen($charset);
-        $txt_len = strlen($text) * 2;
+        $encoded = trim(base64_encode($text));
+        $c_size = strlen($charset) + 7;
 
-        /* RFC 2047 [2] states that no encoded word can be more than 75
-           characters long. If longer, you must split the word. */
-        if (($txt_len + $char_len + 7) > 75) {
-            $pos = intval((68 - $char_len) / 2);
-            return MIME::_encode(substr($text, 0, $pos), $charset) . ' ' . MIME::_encode(substr($text, $pos), $charset);
+        if ((strlen($encoded) + $c_size) > 75) {
+            $parts = explode("\r\n", rtrim(chunk_split($encoded, intval((75 - $c_size) / 4) * 4)));
         } else {
-            return '=?' . $charset . '?b?' . trim(base64_encode($text)) . '?=';
+            $parts[] = $encoded;
         }
+
+        $p_size = count($parts);
+        $out = '';
+        foreach ($parts as $key => $val) {
+            $out .= '=?' . $charset . '?b?' . $val . '?=';
+            if ($p_size > $key + 1) {
+                /* RFC 2047 [2] states that no encoded word can be more than
+                 * 75 characters long. If longer, you must split the word with
+                 * CRLF SPACE. */
+                $out .= "\r\n ";
+            }
+        }
+
+        return $out;
     }
 
     /**
      * Encodes a line via quoted-printable encoding.
      * Wraps lines at 76 characters.
      *
-     * @access public
-     *
      * @param string $text  The text to encode.
      * @param string $eol   The EOL sequence to use.
      *
@@ -195,43 +201,44 @@ class MIME {
      */
     function quotedPrintableEncode($text, $eol)
     {
-        $output = '';
+        $line = $output = '';
+        $curr_length = 0;
 
-        if (Util::extensionExists('imap')) {
-            foreach (preg_split("/\r?\n/", $text) as $line) {
-                $output .= imap_8bit($line) . $eol;
+        /* We need to go character by character through the data. */
+        $length = strlen($text);
+        for ($i = 0; $i < $length; ++$i) {
+            $char = $text[$i];
+
+            /* If we have reached the end of the line, reset counters. */
+            if ($char == "\n") {
+                $output .= $eol;
+                $curr_length = 0;
+                continue;
+            } elseif ($char == "\r") {
+                continue;
             }
-        } else {
-            foreach (preg_split("/\r?\n/", $text) as $line) {
-                /* We need to go character by character through the line. */
-                $length = strlen($line);
-                $current_line = '';
-                $current_length = 0;
 
-                for ($i = 0; $i < $length; $i++) {
-                    $char = substr($line, $i, 1);
-                    $ascii = ord($char);
+            $ascii = ord($char);
+            $char_len = 1;
 
-                    /* Spaces or tabs at the end of the line are NOT allowed.
-                     * Also, Characters in ASCII below 32 or above 126 AND 61
-                     * must be encoded. */
-                    if (((($ascii === 9) || ($ascii === 32)) && ($i == ($length - 1))) ||
-                        (($ascii < 32) || ($ascii > 126) || ($ascii === 61))) {
-                        $char = '=' . String::upper(sprintf('%02s', dechex($ascii)));
-                    }
-
-                    /* Lines must be 76 characters or less. */
-                    $char_length = strlen($char);
-                    $current_length += $char_length;
-                    if ($current_length > 76) {
-                        $output .= $current_line . '=' . $eol;
-                        $current_line = '';
-                        $current_length = $char_length;
-                    }
-                    $current_line .= $char;
-                }
-                $output .= $current_line . $eol;
+            /* Spaces or tabs at the end of the line are NOT allowed. Also,
+             * ASCII characters below 32 or above 126 AND 61 must be
+             * encoded. */
+            if ((($ascii === 32) &&
+                 ($i + 1 != $length) &&
+                 (($text[$i + 1] == "\n") || ($text[$i + 1] == "\r"))) ||
+                (($ascii < 32) || ($ascii > 126) || ($ascii === 61))) {
+                $char_len = 3;
+                $char = '=' . String::upper(sprintf('%02s', dechex($ascii)));
             }
+
+            /* Lines must be 76 characters or less. */
+            $curr_length += $char_len;
+            if ($curr_length > 76) {
+                $output .= '=' . $eol;
+                $curr_length = $char_len;
+            }
+            $output .= $char;
         }
 
         return $output;
@@ -243,60 +250,64 @@ class MIME {
      * This differs from MIME::encode() because it keeps email addresses legal,
      * only encoding the personal information.
      *
-     * @access public
-     *
-     * @param string $text       The email addresses to encode.
+     * @param string $addresses  The email addresses to encode.
      * @param string $charset    The character set of the text.
      * @param string $defserver  The default domain to append to mailboxes.
      *
      * @return string  The text, encoded only if it contains non-ascii
-     *                 characters.
+     *                 characters, or PEAR_Error on error.
      */
-    function encodeAddress($text, $charset = null, $defserver = null)
+    function encodeAddress($addresses, $charset = null, $defserver = null)
     {
-        /* parseAddressList() does not process the null entry
-         * 'undisclosed-recipients:;' correctly. */
-        if (preg_match('/undisclosed-recipients:\s*;/i', trim($text))) {
-            return $text;
-        }
+        if (is_array($addresses)) {
+            $addr_arr = $addresses;
+        } else {
+            /* parseAddressList() does not process the null entry
+             * 'undisclosed-recipients:;' correctly. */
+            if (preg_match('/undisclosed-recipients:\s*;/i', trim($addresses))) {
+                return $addresses;
+            }
 
-        require_once 'Mail/RFC822.php';
-        $parser = &new Mail_RFC822();
-        $addr_arr = $parser->parseAddressList($text, $defserver, false, false);
-        $text = '';
-
-        if (is_array($addr_arr)) {
-            foreach ($addr_arr as $addr) {
-                if (empty($addr->personal)) {
-                    $personal = '';
-                } else {
-                    if ((substr($addr->personal, 0, 1) == '"') &&
-                        (substr($addr->personal, -1) == '"')) {
-                        $addr->personal = substr($addr->personal, 1, -1);
-                    }
-                    $personal = MIME::encode($addr->personal, $charset);
-                }
-                if (strlen($text) != 0) {
-                    $text .= ', ';
-                }
-                $text .= MIME::trimEmailAddress(MIME::rfc822WriteAddress($addr->mailbox, $addr->host, $personal));
+            require_once 'Mail/RFC822.php';
+            $parser = &new Mail_RFC822();
+            $addr_arr = $parser->parseAddressList($addresses, $defserver, true, false);
+            if (is_a($addr_arr, 'PEAR_Error')) {
+                return $addr_arr;
             }
         }
 
-        return $text;
+        $text = '';
+        if (is_array($addr_arr)) {
+            foreach ($addr_arr as $addr) {
+                // Check for groups.
+                if (!empty($addr->groupname)) {
+                    $text .= MIME::encode($addr->groupname, $charset) . ': ' . MIME::encodeAddress($addr->addresses) . '; ';
+                } else {
+                    if (empty($addr->personal)) {
+                        $personal = '';
+                    } else {
+                        if ((substr($addr->personal, 0, 1) == '"') &&
+                            (substr($addr->personal, -1) == '"')) {
+                            $addr->personal = stripslashes(substr($addr->personal, 1, -1));
+                        }
+                        $personal = MIME::encode($addr->personal, $charset);
+                    }
+                    $text .= MIME::trimEmailAddress(MIME::rfc822WriteAddress($addr->mailbox, $addr->host, $personal)) . ', ';
+                }
+            }
+        }
+
+        return rtrim($text, ' ,');
     }
 
     /**
      * Decodes an RFC 2047-encoded string.
      *
-     * @access public
-     *
      * @param string $string      The text to decode.
      * @param string $to_charset  The charset that the text should be decoded
      *                            to.
      *
-     * @return string  The decoded text, or the original string if it was not
-     *                 encoded.
+     * @return string  The decoded text.
      */
     function decode($string, $to_charset = null)
     {
@@ -305,7 +316,7 @@ class MIME {
         }
 
         /* Take out any spaces between multiple encoded words. */
-        $string = preg_replace('|\?=\s=\?|', '?==?', $string);
+        $string = preg_replace('|\?=\s+=\?|', '?==?', $string);
 
         /* Save any preceding text. */
         $preceding = substr($string, 0, $pos);
@@ -335,7 +346,8 @@ class MIME {
         $encoded_text = substr($search, 0, $end);
         $rest = substr($string, (strlen($preceding . $charset . $encoding . $encoded_text) + 6));
 
-        if (!isset($to_charset)) {
+        if (is_null($to_charset)) {
+            require_once 'Horde/NLS.php';
             $to_charset = NLS::getCharset();
         }
 
@@ -361,17 +373,89 @@ class MIME {
         return $preceding . $decoded . MIME::decode($rest, $to_charset);
     }
 
+    /**
+     * Decodes an RFC 2047-encoded address string.
+     *
+     * @param string $string      The text to decode.
+     * @param string $to_charset  The charset that the text should be decoded
+     *                            to.
+     *
+     * @return string  The decoded text.
+     */
+    function decodeAddrString($string, $to_charset = null)
+    {
+        $addr_list = array();
+        foreach (MIME::parseAddressList($string) as $ob) {
+            $ob->personal = isset($ob->personal) ? MIME::decode($ob->personal, $to_charset) : '';
+            $addr_list[] = $ob;
+        }
+
+        return MIME::addrArray2String($addr_list);
+    }
+
+    /**
+     * Encodes a string pursuant to RFC 2231.
+     *
+     * @param string $name     The parameter name.
+     * @param string $string   The string to encode.
+     * @param string $charset  The charset the text should be encoded with.
+     * @param string $lang     The language to use when encoding.
+     *
+     * @return array  The encoded parameter string.
+     */
+    function encodeRFC2231($name, $string, $charset, $lang = null)
+    {
+        $encode = $wrap = false;
+        $output = array();
+
+        if (MIME::is8bit($string, $charset)) {
+            $string = String::lower($charset) . '\'' . (($lang === null) ? '' : String::lower($lang)) . '\'' . rawurlencode($string);
+            $encode = true;
+        }
+
+        // 4 = '*', 2x '"', ';'
+        $pre_len = strlen($name) + 4 + (($encode) ? 1 : 0);
+        if (($pre_len + strlen($string)) > 76) {
+            while ($string) {
+                $chunk = 76 - $pre_len;
+                $pos = min($chunk, strlen($string) - 1);
+                if (($chunk == $pos) && ($pos > 2)) {
+                    for ($i = 0; $i <= 2; $i++) {
+                        if ($string[$pos-$i] == '%') {
+                            $pos -= $i + 1;
+                            break;
+                        }
+                    }
+                }
+                $lines[] = substr($string, 0, $pos + 1);
+                $string = substr($string, $pos + 1);
+            }
+            $wrap = true;
+        } else {
+            $lines = array($string);
+        }
+
+        $i = 0;
+        foreach ($lines as $val) {
+            $output[] =
+                $name .
+                (($wrap) ? ('*' . $i++) : '') .
+                (($encode) ? '*' : '') .
+                '="' . $val . '"';
+        }
+
+        return implode('; ', $output);
+    }
+
     /**
      * Decodes an RFC 2231-encoded string.
      *
-     * @access public
-     *
-     * @param string $string   The entire string to decode, including the
-     *                         parameter name.
-     * @param string $charset  The charset that the text should be decoded to.
+     * @param string $string      The entire string to decode, including the
+     *                            parameter name.
+     * @param string $to_charset  The charset the text should be decoded to.
      *
      * @return array  The decoded text, or the original string if it was not
-     *                 encoded.
+     *                encoded.
      */
     function decodeRFC2231($string, $to_charset = null)
     {
@@ -380,6 +464,7 @@ class MIME {
         }
 
         if (!isset($to_charset)) {
+            require_once 'Horde/NLS.php';
             $to_charset = NLS::getCharset();
         }
 
@@ -387,7 +472,7 @@ class MIME {
         $charset = $lang = null;
         $output = '';
 
-        /* Get the characer set and language used in the encoding, if
+        /* Get the character set and language used in the encoding, if
          * any. */
         if (preg_match("/^[^=]+\*\=([^']*)'([^']*)'/", $string, $matches)) {
             $charset = $matches[1];
@@ -395,7 +480,7 @@ class MIME {
             $string = str_replace($charset . "'" . $lang . "'", '', $string);
         }
 
-        $lines = preg_split('/' . preg_quote($attribute) . '(?:\*\d)*/', $string);
+        $lines = preg_split('/\s*' . preg_quote($attribute) . '(?:\*\d)*/', $string);
         foreach ($lines as $line) {
             $pos = strpos($line, '*=');
             if ($pos === 0) {
@@ -424,8 +509,6 @@ class MIME {
      * If an email address has no personal information, get rid of any angle
      * brackets (<>) around it.
      *
-     * @access public
-     *
      * @param string $address  The address to trim.
      *
      * @return string  The trimmed address.
@@ -444,8 +527,6 @@ class MIME {
     /**
      * Builds an RFC 822 compliant email address.
      *
-     * @access public
-     *
      * @param string $mailbox   Mailbox name.
      * @param string $host      Domain name of mailbox's host.
      * @param string $personal  Personal name phrase.
@@ -457,20 +538,19 @@ class MIME {
     {
         $address = '';
 
-        if (!empty($personal)) {
-            $vars = get_class_vars('MIME');
-            $address .= MIME::_rfc822Encode($personal, $vars['rfc822_filter'] . '.');
+        if (strlen($personal)) {
+            $address .= MIME::_rfc2822Encode($personal, 'personal');
             $address .= ' <';
         }
 
         if (!is_null($host)) {
-            $address .= MIME::_rfc822Encode($mailbox);
+            $address .= MIME::_rfc2822Encode($mailbox, 'address');
             if (substr($host, 0, 1) != '@') {
                 $address .= '@' . $host;
             }
         }
 
-        if (!empty($personal)) {
+        if (strlen($personal)) {
             $address .= '>';
         }
 
@@ -478,60 +558,67 @@ class MIME {
     }
 
     /**
-     * Explodes a RFC 822 string, ignoring a delimiter if preceded by a "\"
-     * character, or if delimiter is inside single or double quotes.
+     * Explodes an RFC 2822 string, ignoring a delimiter if preceded
+     * by a "\" character, or if the delimiter is inside single or
+     * double quotes.
      *
-     * @access public
-     *
-     * @param string $str        The RFC 822 string.
-     * @param string $delimiter  The delimter.
+     * @param string $string     The RFC 822 string.
+     * @param string $delimiters A string containing valid delimiters.
+     *                           Defaults to ','.
      *
      * @return array  The exploded string in an array.
      */
-    function rfc822Explode($str, $delimiter)
+    function rfc822Explode($string, $delimiters = ',')
     {
-        $arr = array();
-        $match = 0;
-        $quotes = array('"', "'");
-        $in_quote = null;
+        $emails = array();
+        $pos = 0;
+        $in_group = $in_quote = false;
         $prev = null;
 
-        if (empty($str)) {
-            return array($str);
+        if (!strlen($string)) {
+            return array($string);
         }
 
-        if (in_array($str[0], $quotes)) {
-            $in_quote = $str[0];
-        } elseif ($str[0] == $delimiter) {
-            $arr[] = '';
-            $match = 1;
+        $char = $string[0];
+        if ($char == '"') {
+            $in_quote = true;
+        } elseif ($char == ':') {
+            $in_group = true;
+        } elseif (strpos($delimiters, $char) !== false) {
+            $emails[] = '';
+            $pos = 1;
         }
 
-        for ($i = 1; $i < strlen($str); $i++) {
-            $char = $str{$i};
-            if (in_array($char, $quotes)) {
+        for ($i = 1, $iMax = strlen($string); $i < $iMax; ++$i) {
+            $char = $string[$i];
+            if ($char == '"') {
                 if ($prev !== '\\') {
-                    if ($in_quote === $char) {
-                        $in_quote = null;
-                    } elseif (is_null($in_quote)) {
-                        $in_quote = $char;
-                    }
+                    $in_quote = !$in_quote;
+                }
+            } elseif ($in_group) {
+                if ($char == ';') {
+                    $emails[] = substr($string, $pos, $i - $pos + 1);
+                    $pos = $i + 1;
+                    $in_group = false;
+                }
+            } elseif (!$in_quote) {
+                if ($char == ':') {
+                    $in_group = true;
+                } elseif (strpos($delimiters, $char) !== false &&
+                          $prev !== '\\') {
+                    $emails[] = substr($string, $pos, $i - $pos);
+                    $pos = $i + 1;
                 }
-            } elseif (($char == $delimiter) &&
-                      ($prev !== '\\') &&
-                      is_null($in_quote)) {
-                $arr[] = substr($str, $match, $i - $match);
-                $match = $i + 1;
             }
             $prev = $char;
         }
 
-        if ($match != $i) {
-            /* The string ended without a $delimiter. */
-            $arr[] = substr($str, $match, $i - $match);
+        if ($pos != $i) {
+            /* The string ended without a delimiter. */
+            $emails[] = substr($string, $pos, $i - $pos);
         }
 
-        return $arr;
+        return $emails;
     }
 
     /**
@@ -545,8 +632,6 @@ class MIME {
      *   $object->host     = The host the mailbox is on ("example.com")
      * </pre>
      *
-     * @access public
-     *
      * @param stdClass $ob   The address object to be turned into a string.
      * @param mixed $filter  A user@example.com style bare address to ignore.
      *                       Either single string or an array of strings.  If
@@ -599,8 +684,6 @@ class MIME {
      * Takes an array of address objects, as returned by imap_headerinfo(),
      * for example, and passes each of them through MIME::addrObject2String().
      *
-     * @access public
-     *
      * @param array $addresses  The array of address objects.
      * @param mixed $filter     A user@example.com style bare address to
      *                          ignore.  If any address matches $filter, it
@@ -621,7 +704,7 @@ class MIME {
             $val = MIME::addrObject2String($addr, $filter);
             if (!empty($val)) {
                 $bareAddr = String::lower(MIME::bareAddress($val));
-                if (!array_key_exists($bareAddr, $addrList)) {
+                if (!isset($addrList[$bareAddr])) {
                     $addrList[$bareAddr] = $val;
                 }
             }
@@ -637,8 +720,6 @@ class MIME {
     /**
      * Returns the bare address.
      *
-     * @access public
-     *
      * @param string $address    The address string.
      * @param string $defserver  The default domain to append to mailboxes.
      * @param boolean $multiple  Should we return multiple results?
@@ -651,15 +732,15 @@ class MIME {
     {
         $addressList = array();
 
-        /* Don't use built-in IMAP function (imap_rfc822_parse_adrlist)
-         * because it doesn't parse distribution lists properly. */
-        require_once 'Mail/RFC822.php';
-        $parser = &new Mail_RFC822();
-        $from = $parser->parseAddressList($address, $defserver, false, false);
+        $from = MIME::parseAddressList($address, $defserver);
+        if (is_a($from, 'PEAR_Error')) {
+            return $multiple ? array() : '';
+        }
 
         foreach ($from as $entry) {
             if (isset($entry->mailbox) &&
-                ($entry->mailbox != 'undisclosed-recipients')) {
+                $entry->mailbox != 'undisclosed-recipients' &&
+                $entry->mailbox != 'UNEXPECTED_DATA_AFTER_ADDRESS') {
                 if (isset($entry->host)) {
                     $addressList[] = $entry->mailbox . '@' . $entry->host;
                 } else {
@@ -668,38 +749,88 @@ class MIME {
             }
         }
 
-        if ($multiple) {
-            return $addressList;
+        return $multiple ? $addressList : array_pop($addressList);
+    }
+
+    /**
+     * Parses a list of email addresses into its parts.
+     *
+     * Works with and without the imap extension being available and parses
+     * distribution lists as well.
+     *
+     * @since Horde 3.2
+     * @see   http://www.php.net/imap_rfc822_parse_adrlist
+     *
+     * @param string $address    The address string.
+     * @param string $defserver  The default domain to append to mailboxes.
+     * @param boolean $validate  Whether to validate the address(es).
+     *
+     * @return array  A list of objects with the possible properties 'mailbox',
+     *                'host', 'personal', 'adl', and 'comment'.
+     */
+    function parseAddressList($address, $defserver = null, $validate = false)
+    {
+        if (preg_match('/undisclosed-recipients:\s*;/i', trim($address))) {
+            return array();
+        }
+
+        /* Use built-in IMAP function only if available and if not parsing
+         * distribution lists because it doesn't parse distribution lists
+         * properly. */
+        if (!$validate &&
+            strpos($address, ':') === false &&
+            Util::extensionExists('imap')) {
+            return imap_rfc822_parse_adrlist($address, $defserver);
         } else {
-            return array_pop($addressList);
+            static $parser;
+            if (!isset($parser)) {
+                require_once 'Mail/RFC822.php';
+                $parser = new Mail_RFC822();
+            }
+            $ret = $parser->parseAddressList($address, $defserver, false,
+                                             $validate);
+            return is_a($ret, 'PEAR_Error') ? array() : $ret;
         }
     }
 
     /**
-     * Quotes and escapes the given string if necessary.
+     * Quotes and escapes the given string if necessary using rules contained
+     * in RFC 2822 [3.2.5].
      *
      * @access private
      *
-     * @param string $str     The string to be quoted and escaped.
-     * @param string $filter  A list of characters that make it necessary to
-     *                        quote the string if they occur.
+     * @param string $str  The string to be quoted and escaped.
+     * @param string $type  Either 'address' or 'personal'.
      *
      * @return string  The correctly quoted and escaped string.
      */
-    function _rfc822Encode($str, $filter = '')
+    function _rfc2822Encode($str, $type = 'address')
     {
-        if (empty($filter)) {
-            $vars = get_class_vars('MIME');
-            $filter = $vars['rfc822_filter'] . ' ';
+        // Excluded (in ASCII): 0-8, 10-31, 34, 40-41, 44, 58-60, 62, 64,
+        // 91-93, 127
+        $filter = "\0\1\2\3\4\5\6\7\10\12\13\14\15\16\17\20\21\22\23\24\25\26\27\30\31\32\33\34\35\36\37\"(),:;<>@[\\]\177";
+
+        switch ($type) {
+        case 'address':
+            // RFC 2822 [3.4.1]: (HTAB, SPACE) not allowed in address
+            $filter .= "\11\40";
+            break;
+
+        case 'personal':
+            // RFC 2822 [3.4]: Period not allowed in display name
+            $filter .= '.';
+            break;
         }
 
         // Strip double quotes if they are around the string already.
+        // If quoted, we know that the contents are already escaped, so
+        // unescape now.
         if (substr($str, 0, 1) == '"' && substr($str, -1) == '"') {
-            $str = substr($str, 1, -1);
+            $str = stripslashes(substr($str, 1, -1));
         }
 
         if (strcspn($str, $filter) != strlen($str)) {
-            return '"' . str_replace('"', '\\"', str_replace('\\', '\\\\', $str)) . '"';
+            return '"' . addcslashes($str, '\\"') . '"';
         } else {
             return $str;
         }
@@ -708,8 +839,6 @@ class MIME {
     /**
      * Returns the MIME type for the given input.
      *
-     * @access public
-     *
      * @param mixed $input     Either the MIME code or type string.
      * @param integer $format  If MIME_CODE, return code.
      *                         If MIME_STRING, returns lowercase string.
@@ -724,8 +853,6 @@ class MIME {
     /**
      * Returns the MIME encoding for the given input.
      *
-     * @access public
-     *
      * @param mixed $input     Either the MIME code or encoding string.
      * @param integer $format  If MIME_CODE, return code.
      *                         If MIME_STRING, returns lowercase string.
@@ -754,6 +881,9 @@ class MIME {
     function _getCode($input, $format, $type)
     {
         $numeric = is_numeric($input);
+        if (!$numeric) {
+            $input = String::lower($input);
+        }
 
         switch ($format) {
         case MIME_CODE:
@@ -761,7 +891,7 @@ class MIME {
             break;
 
         case MIME_STRING:
-            if (!$numeric) return String::lower($input);
+            if (!$numeric) return $input;
             break;
         }
 
@@ -769,7 +899,7 @@ class MIME {
 
         if ($numeric) {
             if (isset($vars[$type][$input])) {
-                return String::lower($vars[$type][$input]);
+                return $vars[$type][$input];
             }
         } else {
             if (($search = array_search($input, $vars[$type]))) {
@@ -784,31 +914,19 @@ class MIME {
      * Generates a Message-ID string conforming to RFC 2822 [3.6.4] and the
      * standards outlined in 'draft-ietf-usefor-message-id-01.txt'.
      *
-     * @access public
-     *
      * @param string  A message ID string.
      */
     function generateMessageID()
     {
-        return '<' . date('YmdHis') . '.' .
-            substr(base_convert(microtime() . mt_rand(), 10, 36), -16) .
-            '@' . $_SERVER['SERVER_NAME'] . '>';
+        return '<' . date('YmdHis') . '.'
+            . substr(str_pad(base_convert(microtime(), 10, 36), 16, uniqid(mt_rand()), STR_PAD_LEFT), -16)
+            . '@' . $_SERVER['SERVER_NAME'] . '>';
     }
 
     /**
      * Adds proper linebreaks to a header string.
-     * RFC 2822 [2.2.3] says that headers SHOULD be wrapped at 78 characters.
-     * However, since we may be dealing with quoted text that contains spaces,
-     * strict wrapping at 78 characters may add unwanted newlines and tabs to
-     * the quoted parameter.  The correct way to get around this is to encode
-     * paraeters according to RFC 2231.  However, since most mailers do not
-     * seem to support this, we will instead simply use long lines.  RFC 2822
-     * says headers SHOULD only be 78 characters a line, but also says that
-     * a header line MUST not be more than 998 characters.  Therefore, the
-     * compromise is to put each parameter on a separate line and chop it
-     * only if it is longer than 998 characters.
-     *
-     * @access public
+     * RFC 2822 says headers SHOULD only be 78 characters a line, but also
+     * says that a header line MUST not be more than 998 characters.
      *
      * @param string $header  The header name.
      * @param string $text    The text of the header field.
@@ -819,10 +937,10 @@ class MIME {
     function wrapHeaders($header, $text, $eol = "\r\n")
     {
         $header = rtrim($header);
+        $text = rtrim($text);
 
         /* Remove any existing linebreaks. */
-        $text = preg_replace("/\r?\n\s?/", ' ', $text);
-        $text = $header . ': ' . rtrim($text);
+        $text = $header . ': ' . preg_replace("/\r?\n\s?/", ' ', $text);
 
         $eollength = strlen($eol);
         $header_lower = strtolower($header);
@@ -830,10 +948,10 @@ class MIME {
         if (($header_lower != 'content-type') &&
             ($header_lower != 'content-disposition')) {
             /* Wrap the line. */
-            $line = wordwrap($text, 75, $eol . "\t");
+            $line = wordwrap($text, 75, $eol . ' ');
 
             /* Make sure there are no empty lines. */
-            $line = preg_replace('/' . $eol . "\t\s*" . $eol . "\t/", '/' . $eol . "\t/", $line);
+            $line = preg_replace('/' . $eol . ' ' . $eol . ' /', '/' . $eol . " /", $line);
 
             return substr($line, strlen($header) + 2);
         }
@@ -845,7 +963,8 @@ class MIME {
         $length = 1000 - $eollength;
         $paramcount = count($params);
 
-        foreach ($params as $count => $val) {
+        reset($params);
+        while (list($count, $val) = each($params)) {
             /* If longer than RFC allows, then simply chop off the excess. */
             $moreparams = (($count + 1) != $paramcount);
             $maxlength = $length - (!empty($line) ? 1 : 0) - (($moreparams) ? 1 : 0);
@@ -858,11 +977,10 @@ class MIME {
                     $val = substr($val, 0, -1);
                     $val .= '"';
                 }
-
             }
 
             if (!empty($line)) {
-                $line .= "\t";
+                $line .= ' ';
             }
             $line .= $val . (($moreparams) ? ';' : '') . $eol;
         }
