@@ -8,23 +8,24 @@ require_once 'Horde/String.php';
  * levels.
  *
  * CSS class names called "quoted1" ... "quoted{$cssLevels}" must be present.
+ * CSS class name "togglequote" is used to style toggle text.
  * The text to be passed in must have already been passed through
  * htmlspecialchars().
  *
  * Parameters:
  * <pre>
- * 'citeblock'  -- Display cite blocks? Defaults to true.
- * 'cssLevels'  -- Number of defined CSS class names. Defaults to 5.
- * 'hideBlocks' -- Hide quoted text blocks by default? Defaults to false.
+ * 'citeblock'  -- Display cite blocks? (DEFAULT: true)
+ * 'cssLevels'  -- Number of defined CSS class names. (DEFAULT: 5)
+ * 'hideBlocks' -- Hide quoted text blocks by default? (DEFAULT: false)
+ * 'outputJS'   -- Add necessary JS files? (DEFAULT: true)
  * </pre>
  *
- * $Horde: framework/Text_Filter/Filter/highlightquotes.php,v 1.6.8.16 2006/08/16 16:56:00 slusarz Exp $
+ * $Horde: framework/Text_Filter/Filter/highlightquotes.php,v 1.6.8.23 2008/10/03 19:28:35 slusarz Exp $
  *
- * Copyright 2004-2006 Michael Slusarz <slusarz@curecanti.org>
- * Copyright 2004-2006 Jan Schneider <jan@horde.org>
+ * Copyright 2004-2008 The Horde Project (http://www.horde.org/)
  *
- * See the enclosed file COPYING for license information (LGPL). If you did
- * not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
+ * See the enclosed file COPYING for license information (LGPL). If you
+ * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Michael Slusarz <slusarz@curecanti.org>
  * @author  Jan Schneider <jan@horde.org>
@@ -41,7 +42,8 @@ class Text_Filter_highlightquotes extends Text_Filter {
     var $_params = array(
         'citeblock' => true,
         'cssLevels' => 5,
-        'hideBlocks' => false
+        'hideBlocks' => false,
+        'outputJS' => true
     );
 
     /**
@@ -68,11 +70,9 @@ class Text_Filter_highlightquotes extends Text_Filter {
     {
         /* Remove extra spaces before quoted text as the CSS formatting will
          * automatically add a bit of space for us. */
-        if ($this->_params['citeblock']) {
-            return array('regexp' => array("/<br \/>\s*\n\s*<br \/>\s*\n\s*((&gt;\s?)+)/m" => "<br />\n\\1"));
-        }
-
-        return array();
+        return ($this->_params['citeblock'])
+            ? array('regexp' => array("/<br \/>\s*\n\s*<br \/>\s*\n\s*((&gt;\s?)+)/m" => "<br />\n\\1"))
+            : array();
     }
 
     /**
@@ -96,9 +96,19 @@ class Text_Filter_highlightquotes extends Text_Filter {
         /* Generated output. */
         $text_out = '';
 
+        /* As of PHP 5.2, backtrack limits have been set to an unreasonably
+         * low number. The body check will often times trigger backtrack
+         * errors so up the backtrack limit if we are doing this match. */
+        if (ini_get('pcre.backtrack_limit')) {
+            ini_set('pcre.backtrack_limit', 5000000);
+        }
+
         /* Add show/hide toggles for whole quote blocks. */
-        $text = preg_replace_callback('/(\n)(( *(&gt;\s?)+(?! ?&gt;).*?)(\n|$)(?! *(&gt; ?)+))/s',
-                                      array($this, '_addQuoteToggles'), $text);
+        $text = preg_replace_callback(
+            '/(\n)(( *(&gt;\s?)+(?! ?&gt;).*?)(\n|$)(?! *(&gt; ?)+))/s',
+            array($this, '_addQuoteToggles'), $text);
+
+        ini_restore('pcre.backtrack_limit');
 
         /* Parse text line by line. */
         foreach (explode("\n", $text) as $line) {
@@ -130,10 +140,10 @@ class Text_Filter_highlightquotes extends Text_Filter {
                     $text_out .= ($cb) ? '</div>' : '</font>';
                     --$closelevel;
                 }
-                /* Strip trailing line break if using cite blocks, we already
-                 * have a div tag. */
-                if ($cb && preg_match('/\<br \/>$/', $line)) {
-                    $line = substr($line, 0, -6);
+                /* Strip trailing line break(s) if using cite blocks, we
+                 * already have a div tag. */
+                if ($cb) {
+                    $line = preg_replace('/(\<br\s*\/>\s*)+(\s*)$/Ui', '$2', $line);
                 }
             /* Is this cite level higher than the current level? */
             } elseif ($clevel > $qlevel) {
@@ -181,8 +191,6 @@ class Text_Filter_highlightquotes extends Text_Filter {
      */
     function _addQuoteToggles($matches)
     {
-        static $i = 0;
-
         /* Don't toggle small blocks; doesn't provide a UI benefit and looks
          * annoying. */
         $lines = substr_count($matches[0], "\n");
@@ -190,21 +198,16 @@ class Text_Filter_highlightquotes extends Text_Filter {
             return $matches[0];
         }
 
-        Horde::addScriptFile('hideable.js', 'horde', true);
-        Horde::addScriptFile('quoteBlocks.js', 'horde');
+        if ($this->_params['outputJS']) {
+            Horde::addScriptFile('prototype.js', 'horde', true);
+        }
 
-        $text = '';
-        if ($this->_params['citeblock']) {
-            $text = '<br />';
-        };
-        $text .= '<div id="qt_' . $i . '">' .
-            Horde::link('#', '', 'widget togglequote', '', 'toggleQuoteBlock(\'' . $i . '\', \'' . $lines . '\'); return false;', '', '', array('style' => 'font-size:70%')) .
-            ($this->_params['hideBlocks'] ? sprintf(_("[Show Quoted Text - %s lines]"), $lines) : _("[Hide Quoted Text]")) .
-            '</a></div><div id="qb_' . $i . '"' . ($this->_params['hideBlocks'] ? ' style="display:none;"' : '') . '>' . $matches[0] . '</div>';
-
-        $i++;
-
-        return $text;
+        return (($this->_params['citeblock']) ? '<br />' : '') .
+            '<div class="togglequoteparent">' .
+            '<span onclick="[ this, this.next(), this.next(1) ].invoke(\'toggle\')" class="widget togglequote"' . ($this->_params['hideBlocks'] ? '' : ' style="display:none"') . '>' . htmlspecialchars(sprintf(_("[Show Quoted Text - %d lines]"), $lines)) . '</span>' .
+            '<span onclick="[ this, this.previous(), this.next() ].invoke(\'toggle\')" class="widget togglequote"' . ($this->_params['hideBlocks'] ? ' style="display:none"' : '') . '>' . htmlspecialchars(_("[Hide Quoted Text]")) . '</span>' .
+            '<span' . ($this->_params['hideBlocks'] ? ' style="display:none"' : '') . '>' . $matches[0] . '</span>' .
+            '</div>';
     }
 
 }
