@@ -2,15 +2,14 @@
 /**
  * Class representing vTodos.
  *
- * $Horde: framework/iCalendar/iCalendar/vtodo.php,v 1.13.10.1 2005/01/03 12:19:23 jan Exp $
+ * $Horde: framework/iCalendar/iCalendar/vtodo.php,v 1.13.10.8 2008/07/03 08:42:58 jan Exp $
  *
- * Copyright 2003-2005 Mike Cochrane <mike@graftonhall.co.nz>
+ * Copyright 2003-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Mike Cochrane <mike@graftonhall.co.nz>
- * @version $Revision: 1.13.10.1 $
  * @since   Horde 3.0
  * @package Horde_iCalendar
  */
@@ -21,11 +20,6 @@ class Horde_iCalendar_vtodo extends Horde_iCalendar {
         return 'vTodo';
     }
 
-    function parsevCalendar($data)
-    {
-        parent::parsevCalendar($data, 'VTODO');
-    }
-
     function exportvCalendar()
     {
         return parent::_exportvData('VTODO');
